/*
 *  ==========================================================================
 *  GraphicsGrid.java: Definition and Management of the Graphics Grid....
 *
 *  Written By : Mark Austin                                      October 2005
 *  ==========================================================================
 */

import java.lang.Math.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Color;
import java.awt.font.*;
import java.awt.image.*;
import java.awt.geom.*;   // Needed for affine transformation....
import java.net.URL;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class GraphicsGrid {
    private boolean DEBUG = false;

    // Define default fidelity for grid size....

    private int gridSize     = 50;
    private int smallDotSize =  4;

    // Constructor methods ....

    public GraphicsGrid() {}

    // Method to adjust/set grid size ...

    public void setGridSize ( int gridSize ) {
       this.gridSize = gridSize;

       // Automatically adjust smallDotSize for fine grids ...

       if ( gridSize >= 50 ) 
            smallDotSize = 4;
       else if ( gridSize >= 20 ) 
            smallDotSize = 2;
       else 
            smallDotSize = 1;
    }

    public void drawGrid( Graphics2D g2D, double dMinX, double dMinY, double dMaxX, double dMaxY ) {

       if (DEBUG == true) {
           System.out.printf("*** In GraphicsGrid.drawGrid()\n");
           System.out.printf("***    Min (x,y) = (%3.1f, %3.1f)\n", dMinX, dMinY );
           System.out.printf("***    Max (x,y) = (%3.1f, %3.1f)\n", dMaxX, dMaxY );
       }

       // Draw rectangle around exterior boundary ...

       if (DEBUG == true) {
           g2D.scale( 1.0, -1.0 );
           g2D.drawString("Origin", 0, 0);
           g2D.scale( 1.0, -1.0 );
       }

       Ellipse2D dot = new Ellipse2D.Double( -smallDotSize/2.0, -smallDotSize/2.0,
                                                  smallDotSize,      smallDotSize );

       for (double i = dMinX; i <=  dMaxX; i = i + gridSize )
       for (double j = dMinY; j <=  dMaxY; j = j + gridSize ) {

           // Adjust color for x- and y- axes ....

           if ( i == 0 || j == 0 )
              g2D.setColor( Color.red );
           else
              g2D.setColor( Color.blue );

           // Draw grid dot ....

           g2D.translate(  i, j );
           g2D.scale(  (double) 1.0, (double) -1.0 );
           g2D.fill( dot );
           g2D.scale(  (double) 1.0, (double) -1.0 );
           g2D.translate( -i,-j );
       }
    }

    public void drawBorder( Graphics2D g2D, double dMinX, double dMinY, double dMaxX, double dMaxY ) {

       // Draw rectangle around exterior boundary ...

       g2D.setColor( Color.black );
       g2D.draw( new Line2D.Double( dMinX-10, dMinY-10, dMaxX+10, dMinY-10 ));
       g2D.draw( new Line2D.Double( dMinX-10, dMinY-10, dMinX-10, dMaxY+10 ));
       g2D.draw( new Line2D.Double( dMaxX+10, dMinY-10, dMaxX+10, dMaxY+10 ));
       g2D.draw( new Line2D.Double( dMinX-10, dMaxY+10, dMaxX+10, dMaxY+10 ));
    }
}
