/** 
  *  ======================================================================
  *  SimpleScreen.java. Create simple GUI with panel ...
  * 
  *  Written By : Mark Austin                                  October 2011
  *  ======================================================================
  */ 

import java.lang.Math.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Color;
import java.awt.font.*;
import java.awt.image.*;
import java.awt.geom.*;   // Needed for affine transformation....
import shape.*;           // Import shape classes 

public class SimpleScreen extends JPanel {
    private AffineTransform            at; // save grid transform ....
    private ArrayList<SceneShape>  shapes;
    GraphicsGrid      grid;
    GraphicsOperation grop;
    private Dimension size;
    private Graphics    gs;
    int      width, height;
    boolean  DEBUG = false;

    // Matrix transform for pixel to viewpoint coordinates ....
 
    double[] theMatrix = new double[6];
 
    // Desired coordinate limits.

    private double MinX =  -200.0;
    private double MinY =  -200.0;
    private double MaxX =   500.0;
    private double MaxY =   500.0;

    private int xBorder     = 30;
    private int yBorder     = 30;

    boolean overShape = false;

    //  Constructor for simple screen ....

    SimpleScreen () {
       grid   = new GraphicsGrid();
       grop   = new GraphicsOperation();
       shapes = new ArrayList<SceneShape>();

       CompoundShape car01   = new CarShape(   0,    0,  50);
       car01.setName("Family Wagon");

       CompoundShape car02   = new CarShape( 100,  100, 100);
       car02.setName("Roadster");
       car02.setColor( Color.blue );

       CompoundShape car03   = new CarShape( 300, -100, 150);
       car03.setName("Big Truck");

       CompoundShape car04   = new CarShape( 200, 200,  25);
       car04.setTextOffSetX( 30 );
       car04.setTextOffSetY( 10 );
       car04.setName("Parking");
       CompoundShape car05   = new CarShape( 200, 120,  25);
       car05.setTextOffSetX( 30 );
       car05.setTextOffSetY( 10 );
       car05.setName("Parking");

       CompoundShape house01 = new HouseShape( -200, 400, 150);
       house01.setName("Big house");
       house01.setTextOffSetX( 0 );

       CompoundShape house02 = new HouseShape(  350, 450,  50);
       house02.setName("Vacation cottage");

       CompoundShape circle01 = new CircleShape(  0, 250,  50);
       circle01.setName("Greenbelt Station");
       circle01.setFilledShape( true );
       circle01.setColor( Color.blue );

       CompoundShape circle02 = new CircleShape(  0, 150,  10);
       circle02.setName("College Park Station");
       circle02.setFilledShape( true );
       circle02.setColor( Color.green );

       CompoundShape parking01 = new RectangleShape(  300, 250,  350, 20 );
       parking01.setName("Parking Lot 01");
       parking01.setTextOffSetX(    0 );
       parking01.setTextOffSetY(   -5 );
       parking01.setFilledShape( true );
       parking01.setColor( Color.orange );

       CompoundShape parking02 = new RectangleShape(  100, 350,  100, 300 );
       parking01.setName("Parking Lot 02");
       parking02.setColor( Color.orange );

       // Create a generic polygon shape ....

       double xCoord[] = { 0,  50,   50, 150, 150, 200,   200,    0 };
       double yCoord[] = { 0,   0,  -50, -50,   0,   0,  -100, -100 };
       CompoundShape polygon01 = new PolygonShape( -200, -150, xCoord, yCoord );
       polygon01.setFilledShape( true );
       polygon01.setColor( Color.orange );
       polygon01.setName("AV Williams");
       polygon01.setTextOffSetX(    0 );
       polygon01.setTextOffSetY( -105 );

       // Create a generic track element shape ....

       double dx    =  300.0;
       double dy    = -200.0;
       double width =   10.0;
       CompoundShape track01 = new TrackShape(  125, -50, dx, dy, width );
       track01.setFilledShape( true );
       track01.setColor( Color.green );
       track01.setName("Green Line");

       // Build array of shapes....

       shapes.add( car01 );
       shapes.add( car02 );
       shapes.add( car03 );
       shapes.add( car04 );
       shapes.add( car05 );
       shapes.add( house01 );
       shapes.add( house02 );
       shapes.add( circle01 );
       shapes.add( circle02 );
       shapes.add( parking01 );
       shapes.add( parking02 );
       shapes.add( polygon01 );
       shapes.add( track01 );

       setPreferredSize( new Dimension( 900, 800) );
       setBackground( Color.WHITE );
       setBorder(BorderFactory.createLineBorder( Color.DARK_GRAY, 4) );

       addMouseMotionListener( new MouseMotionAdapter() {
          public void mouseMoved(MouseEvent event) {
             Point mousePoint = event.getPoint();
             boolean newOverShape = false;
             String overShapeName = null;

             // Get point from mouse moved event and covert
             // to grid coordinates ....

             double dx = mousePoint.getX();
             double dy = mousePoint.getY();
             double dVx = dx - xBorder + MinX;
             double dVy = height - yBorder - dy + MinY;

             if ( DEBUG == true ) {
                System.out.printf("*** mouseDragged pixel(x,y)     = (%5.1f, %5.1f\n",  dx, dy );
                System.out.printf("***              viewpoint(x,y) = (%5.1f, %5.1f\n", dVx, dVy );
             }

             newOverShape = false;
             for (SceneShape s : shapes) {
                if ( s.contains( new Point ( (int) dVx, (int) dVy ) ) ) {
                     newOverShape  = true;
                     overShapeName = s.getName();
                }
             }

             if ( DEBUG == true ) {
                 if ( newOverShape == true && overShape == false )
                      System.out.println("*** Over shape = " + overShapeName );
             }

             if ( newOverShape != overShape ) {
                  overShape = newOverShape;
                  repaint();
             } 

          }
        });
    }

    public void paint() {
       Graphics g = getGraphics();
       super.paintComponent(g);
       paintComponent(g);
    }

    public void update(Graphics g) {}

    public void paintComponent(Graphics g) {
        super.paintComponent(g);
        Dimension d = getSize();
        size   = getSize();
        height = getSize().height;
        width  = getSize().width;

        Graphics2D g2D = (Graphics2D) g.create();

        g2D.setRenderingHint( RenderingHints.KEY_ANTIALIASING,
                              RenderingHints.VALUE_ANTIALIAS_ON);
         
        // Transform origin to lower right-hand corner ..

        at = new AffineTransform();
        at.translate(  xBorder, height - yBorder );
        at.scale( 1, -1);
        g2D.setTransform (at);

        // Translate to origin ....

        g2D.translate( -MinX, -MinY );

        // Draw grid .....

        if ( grop.getGrid() == true ) {
             grid.drawGrid( g2D, MinX, MinY, MaxX, MaxY ); 
        }

        // Draw border around grid .....

        if ( grop.getBorder() == true ) {
             grid.drawBorder( g2D, MinX, MinY, MaxX, MaxY ); 
        }

        // Draw scene shapes .....

        g2D.setStroke( new BasicStroke( 2 ));
        for (SceneShape s : shapes) {
           s.draw( g2D );
        }
    }

    // Get graphics and fill in background ....

    public void clearScreen () {
       gs = getGraphics();
       Graphics2D g2D = (Graphics2D) gs;

       g2D.setColor( Color.white );
       g2D.fillRect( 0, 0, size.width-1, size.height-1 );
    }

    // Draw border around graphics screen ....

    void drawBorder() {
       gs = getGraphics();
       Graphics2D g2D = (Graphics2D) gs;

       // Transform origin to lower right-hand corner ..

       AffineTransform at = new AffineTransform();
       at.translate(  xBorder, height - yBorder );
       at.scale( 1, -1);
       g2D.setTransform (at);

       // Translate to origin and draw border ...

       g2D.translate( -MinX, -MinY );
       grid.drawBorder( g2D, MinX, MinY, MaxX, MaxY ); 
    }

    // Draw grid on graphics screen ....

    void drawGrid() {
       grop.setGrid(true);
       gs = getGraphics();
       Graphics2D g2D = (Graphics2D) gs;

       // Transform origin to lower right-hand corner ..

       AffineTransform at = new AffineTransform();
       at.translate(  xBorder, height - yBorder );
       at.scale( 1, -1);
       g2D.setTransform (at);

       // Translate to origin and draw grid ...

       g2D.translate( -MinX, -MinY );
       grid.drawGrid( g2D, MinX, MinY, MaxX, MaxY ); 
    }
}
