/** 
  *  ======================================================================
  *  SimpleScreenToolBar.java: Toolbar for simple GUI...
  * 
  *  Written By: Mark Austin                                  November 2011
  *  ======================================================================
  */ 

import java.lang.Math.*;
import java.applet.*;
import java.util.*;
import java.io.*;
import java.text.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.Color;
import java.awt.font.*;
import java.awt.image.*;
import java.awt.geom.*;   // Needed for affine transformation....
import java.net.URL;

public class SimpleScreenToolBar extends JToolBar {
   SimpleScreen      gs;
   JPopupMenu popupMenu;
   JToolBar   toolBar;
   JMenuBar   menuBar;

   boolean lineEnabled = false;
   boolean rectEnabled = false;

   JDialog dialog = null;
   Color    color = null;

   JButton button1;
   JButton button2;
   JButton button3;
   JButton button4;
   JButton button5;

   String[] taskLabels = {"Add Space",
                          "Add Design Block",
                          "Add Exterior Wall",
                          "Add Primary Space",
                          "Add Room",
                          "Add Portal",
                          "Add Column",
                          "Add Anchor Points",
                          "Label Columns",
                          "Label Walls",
                          "Label Spaces" };

   // Image icons ...

   Icon infoIcon    = new ImageIcon( "icons/info.gif" );
   Icon lineIcon    = new ImageIcon( "icons/line.gif" );
   Icon rectIcon    = new ImageIcon( "icons/rect.gif" );
   Icon paletteIcon = new ImageIcon( "icons/palette.gif" );
   Icon gridIcon    = new ImageIcon( "icons/grid.gif" );

   // Constructor method for application version ....

   SimpleScreenToolBar( SimpleScreen gs ) {
       this.gs = gs;

       // Create items for "file" pull-down menu ....

       JMenuItem openFile = new JMenuItem( "Open" );       
       openFile.addActionListener( new FileOpenListener() );

       JMenuItem saveFile = new JMenuItem( "Save" );       
       saveFile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               System.out.println("*** Selected Save File!");
            }
       });

       JMenuItem saveAsFile = new JMenuItem( "Save As" );       
       saveAsFile.addActionListener(new ActionListener() {
            public void actionPerformed(ActionEvent e) {
               System.out.println("*** Selected Save As File!");
            }
       });

       // Add a file menu to the toolbar ..

       JMenu fileMenu = new JMenu("File", true);
       fileMenu.add( openFile );
       fileMenu.addSeparator();
       fileMenu.add( saveFile );
       fileMenu.add( saveAsFile );
                     
       // Create view submenus ....

       JMenu viewMenu = new JMenu("Layers");
       JCheckBoxMenuItem viewItem;
       String[] lbLabels =  { "Border Layer", "Grid Layer" };
       for (int i=0; i< lbLabels.length; i++) {
          viewItem = new JCheckBoxMenuItem( lbLabels[i], false );
          viewItem.setHorizontalTextPosition( JMenuItem.RIGHT );
          viewItem.addActionListener( new MenuListener() );
          viewMenu.add(viewItem);
       }

       // Create and add the Modify menu and submenus...

       JMenu modifyMenu          = new JMenu("Graphics");
       JMenu graphicsOptionsMenu = new JMenu("Options");
       JMenu gridsizeOptionsMenu = new JMenu("Grid Size");

       modifyMenu.add( graphicsOptionsMenu );
       modifyMenu.add( gridsizeOptionsMenu );

       JCheckBoxMenuItem cbItem;
       String[] cbLabels =  { "Snap to Grid" };
       for (int i=0; i<cbLabels.length; i++) {
            cbItem = new JCheckBoxMenuItem( cbLabels[i] );
            cbItem.addActionListener( new MenuListener() );
            graphicsOptionsMenu.add(cbItem);
       }

       JRadioButtonMenuItem rbItem;
       ButtonGroup group = new ButtonGroup();
       String[] gridLabels = { "5", "10", "20", "30", "40", "50" };
       for (int i=0; i<gridLabels.length; i++) {
            rbItem = new JRadioButtonMenuItem( gridLabels[i] );
            rbItem.addActionListener( new MenuListener() );
            gridsizeOptionsMenu.add(rbItem);
            group.add(rbItem);
       }

       // Add a toolbar to the top of the applet.

       menuBar = new JMenuBar();
       menuBar.setBorder(new BevelBorder(BevelBorder.RAISED));
       menuBar.setBorderPainted(true);

       menuBar.add( fileMenu );
       menuBar.add( viewMenu );
       menuBar.add( modifyMenu );

       this.setLayout( new BorderLayout() );
       this.setMargin( new Insets(5,5,5,5));
       this.setBorder( new BevelBorder(BevelBorder.RAISED) );
       this.setBorderPainted(true);
       this.add( menuBar, BorderLayout.WEST );
   }     

   // Listener for "file open" class ....

   class FileOpenListener implements ActionListener {
       public void actionPerformed(ActionEvent e) {
          System.out.println("*** In FileOpen Listener!");
       }
   }

   // Listener for "draw grid" class ....

   class GridListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {
            JButton button = (JButton) e.getSource();
            if (button.getText().equals("Draw Grid")) {
                gs.grop.setDrawGrid();
                gs.repaint();
            }
        }
   }

   // Listener for handling menu selections ....

   class MenuListener implements ActionListener {
        public void actionPerformed(ActionEvent e) {

            JMenuItem item = (JMenuItem) e.getSource();
            System.out.println("In MenuListener():Label = " + item.getText());

            // Adjust settings for draw box and draw grid ...

            if (item.getText().equals("Border Layer")) {
                gs.grop.setDrawBorder();
                gs.repaint();
            }

            if (item.getText().equals("Grid Layer")) {
                gs.grop.setDrawGrid();
                gs.repaint();
            }

            // Determine which menu item has been selected ....

            if (item.getText().equals("Snap to Grid")) {
                gs.grop.setGridSnap();
            }

            // Set grid size from radio buttons ....

            if ( item.getText().equals( "5") == true ||
                 item.getText().equals("10") == true ||
                 item.getText().equals("20") == true ||
                 item.getText().equals("30") == true ||
                 item.getText().equals("40") == true || 
                 item.getText().equals("50") == true ) { 
                    int iGridSize = Integer.valueOf( item.getText() ).intValue();
                    gs.grid.setGridSize( iGridSize );
                    gs.repaint();
                }
        }
   }     
}
