/**
  *  =====================================================================
  *  PolygonShape.java: Construct a house shape.
  *
  *     @param x the left of the bounding rectangle
  *     @param y the top of the bounding rectangle
  *     @param width the width of the bounding rectangle
  *
  *  =====================================================================
  */

package shape;

import java.awt.*;
import java.awt.geom.*;

public class PolygonShape extends CompoundShape {
   GeneralPath star = new GeneralPath();
   boolean DEBUG = false;

   public PolygonShape( int x, int y, double xPoints[], double yPoints[] ) {

      // Save corner reference point for shape ...

      this.x = (double) x;
      this.y = (double) y;
      this.width  = (double) 0.0;
      this.height = (double) 0.0;

      // set the initial coordinate of the GeneralPath

      star.moveTo( xPoints[ 0 ], yPoints[ 0 ] );
        
      // Create the star--this does not draw the star

      for ( int k = 1; k < xPoints.length; k++ )
         star.lineTo( xPoints[ k ], yPoints[ k ] );
        
      // Close the shape

      star.closePath();

      add(star);
   }

   public boolean contains( Point2D pt ) {
      double xCoord = pt.getX() - this.x;
      double yCoord = this.y - pt.getY();
      Point2D.Double testPoint = new Point2D.Double( xCoord, yCoord );

      // Set shapeActive to true when cursor is inside shape ...

      if ( star.contains ( testPoint ) == true )
         shapeActive = true;  
      else 
         shapeActive = false;  

      if (DEBUG == true) {
         System.out.println("*** In CompoundShape.contains(): name = " + sName );
         System.out.println("***                            : xCoord = " + xCoord );
         System.out.println("***                            : yCoord = " + yCoord );
         System.out.println("***                            : inside = " + shapeActive );
      }

      return star.contains ( testPoint );
   }
}
